#ifndef GUARD_dirdlg_h
#define GUARD_dirdlg_h

// air is little image thumbnail program.
// Copyright (C) 2004 HG, Rhee <ez8tank@krpost.net>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
//
// All comments are korean, UTF-8 encoding.

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#if HAVE_EXT_HASH_SET && __GNUC__ >= 3
#include <ext/hash_set>
#else
#include <set>
#endif

#include <string>

#include <gtkmm/window.h>
#include <gtkmm/treemodel.h>
#include <gtkmm/treestore.h>
#include <gtkmm/treeview.h>
#include <gtkmm/scrolledwindow.h>
#include <gtkmm/separator.h>
#include <gtkmm/buttonbox.h>
#include <gtkmm/button.h>
#include <gtkmm/stock.h>
#include <gtkmm/dialog.h>

class DirBrowseWnd : public Gtk::Window
{
    public:
	DirBrowseWnd();
	~DirBrowseWnd();

	const std::string& SelectedDirectory() { return selectedDirectory_; }
	const Gtk::ResponseType& ResultResponse() { return resultResponse_; }

    private:
	// 트리에 패스가 추가되어있는지 확인하는데 사용할 set
	// 따로 set를 이용하는 쪽이 트리를 직접 검색하는 것보다 약간 빠름
	// 해쉬를 쓸 수 있는 경우에는 사용하도록 함(비표준)
#if HAVE_EXT_HASH_SET && __GNUC__ >= 3
	struct StringHash
	{
	    size_t operator()(const std::string& id) const
	    {
		const std::string::size_type size = id.size();
		size_t h = 0;
		for(std::string::size_type i = 0; i < size; i++)
		{
		    h = 5 * h + id[i];
		}

		return h;
	    }
	};
	
	__gnu_cxx::hash_set<std::string, StringHash> pathInStore_;
#else	
	std::set<std::string> pathInStore_;
#endif
    
	class ModelColumns : public Gtk::TreeModel::ColumnRecord
	{
	    public:
		ModelColumns() { add(colName_); add(colPath_); }

		// 이름쪽은 화면에 표시될 것이므로 Glib::ustring 으로 사용
		// utf8 로 변환시켜 화면에 표시될 것임	
		Gtk::TreeModelColumn<Glib::ustring> colName_;
		Gtk::TreeModelColumn<std::string> colPath_;
	};

	ModelColumns column_;	
    
	Gtk::VBox mainBox_;
	Gtk::ScrolledWindow scrolledWnd_;
	Gtk::TreeView treeView_;
	Gtk::HSeparator btnBoxSeparator_;
	Gtk::HButtonBox mainBtnBox_;
	Gtk::Button cancelBtn_, okBtn_;
	
	// 반환으로 넘겨줄 디렉토리
	static std::string selectedDirectory_;
	// 지금 작업 중인 디렉토리
	std::string currentPath_;
	std::string fullPath_;

	Gtk::ResponseType resultResponse_;
		
	Glib::RefPtr<Gtk::TreeStore> treeModel_;
	
	virtual void on_show();
	
	void OnRowActivated(const Gtk::TreeModel::Path&, Gtk::TreeViewColumn*);
	void OnRowExpanded(const Gtk::TreeModel::iterator&, const Gtk::TreeModel::Path&);

	void OnBtnOk();
	void OnBtnCancel();

	void ExpandCurrentDir();

};

#endif
