#ifndef GUARD_misc_h
#define GUARD_misc_h

// air is little image thumbnail program.
// Copyright (C) 2004 HG, Rhee <ez8tank@krpost.net>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
// All comments are korean, UTF-8 encoding.

#include <sys/stat.h>
#include <sys/types.h>
#include <unistd.h>
#include <dirent.h>

#include <vector>
#include <list>
#include <map>
#include <set>
#include <string>
#include <cctype>

#include <gdkmm.h>
#include <gtkmm/box.h>
#include <gtkmm/label.h>
#include <gtkmm/menu.h>
#include <gtkmm/optionmenu.h>
#include <gtkmm/drawingarea.h>

// string이 한 글자이고 0 혹은 1인지 검사
inline bool IsNumBoolean(const std::string& p)
{
    if(p.size() == 1 && ( ( p[0] == '0')||( p[0] == '1') ) ) 
	return true;
    else 
	return false;    
}

// string의 각 문자가 숫자인지 검사
inline bool IsInteger(const std::string& p)
{
    for(std::string::const_iterator i = p.begin(); i != p.end(); ++i) {
	if(!isdigit(*i)) 
	    return false;
    }

    return true;	
}

inline bool IsDirectory(const std::string& p)
{
    return Glib::file_test(p, Glib::FILE_TEST_IS_DIR);
}

// 알고리즘용 디렉토리 검사 함수
struct EntryIsDirectory: std::binary_function<std::string, std::string, bool> {
    bool operator()(const std::string& path, const std::string& filename) const
    {
	return Glib::file_test(path + "/" + filename, Glib::FILE_TEST_IS_DIR);
    }

};

// 알고리즘용 set<string> 에 해당 string 이 있는지 검사하는 함수
struct EntryInStringSet: std::binary_function<std::set<std::string>, std::string, bool> {
    bool operator()(const std::set<std::string>& origSet, const std::string& testFactor) const
    {
	if(!origSet.count(testFactor))
	    return true;
	else
	    return false;    
    }
};

// 절대경로가 실행할 수 있는지 검사
// Glib::file_test 에도 실행 가능 여부 함수 존재
inline bool IsExecutable(const std::string& p)
{
    return !eaccess(p.c_str(), X_OK);
}

// 절대 경로가 읽을 수 있는지 검사
inline bool IsReadable(const std::string& p)
{
    return !eaccess(p.c_str(), R_OK);
}

void SplitPath(const std::string&, std::vector<std::string>&);

inline std::vector<std::string> SplitPath(const std::string& fullpath)
{
    std::vector<std::string> rtn;
    rtn.reserve(3);

    SplitPath(fullpath, rtn);
    
    return rtn;
}

void SplitEachDir(const std::string&, std::vector<std::string>&);

inline std::vector<std::string> SplitEachDir(const std::string& fullpath)
{
    std::vector<std::string> rtn;
    rtn.reserve(3);

    SplitEachDir(fullpath, rtn);

    return rtn;
}

Glib::ustring StripExt(const Glib::ustring&);
std::string StripName(const std::string&);

void GetFileList(const std::string&, std::list<std::string>&, bool =false);

inline std::list<std::string> GetFileList(const std::string& path, bool recursive = false)
{
    std::list<std::string> rtn;

    GetFileList(path, rtn, recursive);

    return rtn;
}

// 공백 문자인지 검사
struct Space: std::unary_function<char, bool>
{
    bool operator()(const char& p) const
    {
	return isspace(p);	
    }
};    
    
// 등호 기호인지 검사
struct Equal: std::unary_function<char, bool>
{
    bool operator()(const char& p) const
    {
	if(p == '=')
	    return true;
	else
	    return false;
    }
};

// 슬래쉬 기호인지 검사
struct Slash: std::unary_function<char, bool>
{
    bool operator()(const char& p) const
    {
	if(p == '/')
	    return true;
	else
	    return false;    
    }
};

// 한 문자를 소문자로 만듬
struct ToLower: std::unary_function<char, void>
{
    void operator()(char& p) const
    {
	p = std::tolower(p);
    }
};

struct NotInFilterExt: std::binary_function< std::set<std::string>, std::string, bool>
{
    bool operator()(const std::set<std::string>& p1, const std::string &p2) const
    {
	std::string ext = StripName(p2);
	for_each(ext.begin(), ext.end(), ToLower());
	
	// 있으면 false -> remove_if 에서 살아남음
	if(p1.count(ext))
	    return false;
	else
	    // 없으면 true -> remove_if 에서 삭제 
	    return true;	    	
    }
};

class PreviewArea : public Gtk::DrawingArea
{
    public:
	PreviewArea();
	virtual ~PreviewArea();

	void SetImg(Glib::RefPtr<Gdk::Pixbuf>);
	void Draw();

    protected:
	unsigned int imageWidth_;
	unsigned int imageHeight_;
	
	Glib::RefPtr<Gdk::Pixbuf> image_;

	virtual bool on_expose_event(GdkEventExpose* e);

};

bool LoadConfig(std::map<std::string, std::string>&);

class ProcConfigLine
{
    public:
	ProcConfigLine(std::string&);

	std::string key() const { return key_; }
	std::string value() const { return value_; }
	bool valid() const { return valid_; }

    private:
	std::string key_;
	std::string value_;
	bool valid_;

};

#endif
