#ifndef GUARD_preference_h
#define GUARD_preference_h

// air is little image thumbnail program.
// Copyright (C) 2004 HG, Rhee <ez8tank@krpost.net>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
//
// All comments are korean, UTF-8 encoding.

#include <vector>
#include <map>
#include <set>
#include <string>
#include <sstream>
#include <gdkmm.h>

class Preference;
class PrefWnd;

enum FileAddType {
    FILEADD_PREFIX = 0,
    FILEADD_SUFFIX = 1,
    FILEADD_OVERWRITE = 2
};

class Preference {
    public:
	Preference();

	friend void operator<<(Preference&, const PrefWnd&);

	void ConvertFilename(const Glib::ustring&, Glib::ustring&) const;
	Glib::ustring ConvertFilename(const Glib::ustring&) const;

	const bool& StopError() const { return stopError_; }
	const bool& Recursive() const { return recursive_; }
	const bool& ExtFilter() const { return extFilter_; }
	const bool& Preview() const { return preview_; }
	const Gdk::InterpType& InterpType() const { return interpType_; }
	const int& Jpegcomp() const { return jpegcomp_; }
	const FileAddType& FileaddType() const { return fileaddType_; }
	const std::string& FileaddString() const { return fileaddString_; }
	const std::vector<Gdk::PixbufFormat>& ReadableFormats() const { return readableFormats_; }
	const std::vector<Gdk::PixbufFormat>& WritableFormats() const { return writableFormats_; }
	const std::vector<Glib::ustring> JpegOptionKeys() const { return jpegOptionKeys_; }
	const std::vector<Glib::ustring> JpegOptionValues() const 
	{
	    // 생성된 컨테이너는 이미 1개의 요소가 있음
	    std::stringstream buf;
	    buf << jpegcomp_;
	    jpegOptionValues_[0] = buf.str();
	    
	    return jpegOptionValues_;
	}
	const std::set<std::string>& FilterExt() const { return filterExt_; }

    private:
	std::set<std::string> filterExt_;
	std::vector<Gdk::PixbufFormat> readableFormats_;
	std::vector<Gdk::PixbufFormat> writableFormats_;	
	std::vector<Glib::ustring> jpegOptionKeys_;
	mutable std::vector<Glib::ustring> jpegOptionValues_;
	std::string fileaddString_;
	int jpegcomp_;
	Gdk::InterpType interpType_;
	FileAddType fileaddType_;
	bool stopError_;
	bool recursive_;
	bool extFilter_;
	bool preview_;

	template <typename T>
	    void AssignConfig(T&, const std::map<std::string, std::string>&, const std::string&, bool (*f)(const std::string&));

	void AssignConfig(Gdk::InterpType&, const std::map<std::string, std::string>&, const std::string&, bool (*f)(const std::string&));
	void AssignConfig(FileAddType&, const std::map<std::string, std::string>&, const std::string&, bool (*f)(const std::string&));
	void AssignConfig(std::string&, const std::map<std::string, std::string>&, const std::string&);

	struct SeparateWritableFormats
	    : std::binary_function<std::vector<Gdk::PixbufFormat>*, Gdk::PixbufFormat, void>
	    {
		void operator()(std::vector<Gdk::PixbufFormat>* p1, const Gdk::PixbufFormat& p2) const
		{
		    if(p2.is_writable()) p1->push_back(p2);
		}
	    };

	struct InsertStringSet: 
	    std::binary_function<std::set<std::string>*, Gdk::PixbufFormat, void> 
	{
	    void operator()(std::set<std::string>* destSet, const Gdk::PixbufFormat& orig) const
	    {
		std::string name = orig.get_name();
		if(name == "jpeg") {
		    destSet->insert("jpeg");
		    destSet->insert("jpg");
		    destSet->insert("jpe");
		} else if(name == "tiff") {
		    destSet->insert("tiff");
		    destSet->insert("tif");
		} else {
		    destSet->insert(name);
		}
	    }
	};

};

#endif
