// air is little image thumbnail program.
// Copyright (C) 2004 HG, Rhee <ez8tank@krpost.net>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
//
//All comments are korean, UTF-8 encoding.
//
// 간단한 구조 설명
// 1. 맨 처음 루트(/) 추가
// 2. 루트 밑에 dummy 추가
// 3. expander 가 열릴때마다 그 디렉토리의 내용을 읽어와서 추가시킴
// 4. 내용이 추가될 때는 밑에 dummy 추가
//
// 하위 디렉토리 처리에 대한 문제
//  해당 디렉토리에 하위 디렉토리 존재 여부에 관계없이 dummy 를 추가시키고
// expander 가 열릴때 그 내용을 결정시키고 있습니다. 본래대로라면 하위디렉토리
// 가 있는지 없는지 검사를 하고 추가를 시켜주어야 하는데, 디렉토리가 복잡할 수
// 록 걸리는 시간이 상당히 증가하기 때문에 단순히 모두 dummy 를 추가 시키고 있
// 습니다. ( M$윈도우즈의 탐색기의 수상한 행동에서 따라해 봤습니다 :) ) 

#include <vector>
#include <list>
#include <algorithm>

#include <sys/types.h>
#include <unistd.h>
#include <dirent.h>

#include "../include/dirbrowsewin.h"
#include "../include/misc.h"

using std::vector;
using std::list;
using std::string;
using std::sort;

string DirBrowseWnd::selectedDirectory_;

DirBrowseWnd::DirBrowseWnd()
: mainBtnBox_(Gtk::BUTTONBOX_SPREAD),
    cancelBtn_(Gtk::Stock::CANCEL),
    okBtn_(Gtk::Stock::OK),
    resultResponse_(Gtk::RESPONSE_NONE)
{
    set_title("디렉토리를 선택하세요");
    set_border_width(12);
    set_size_request(250, 400);

    add(mainBox_);

    scrolledWnd_.add(treeView_);
    scrolledWnd_.set_policy(Gtk::POLICY_AUTOMATIC, Gtk::POLICY_AUTOMATIC);

    treeModel_ = Gtk::TreeStore::create(column_);
    treeView_.set_model(treeModel_);
    treeView_.append_column("Path", column_.colName_);
    treeView_.signal_row_activated().connect(
	    sigc::mem_fun(*this, &DirBrowseWnd::OnRowActivated));
    treeView_.signal_row_expanded().connect(
	    sigc::mem_fun(*this, &DirBrowseWnd::OnRowExpanded));

    Gtk::TreeModel::Row parent = *(treeModel_->append());
    parent[column_.colName_] = "/";
    parent[column_.colPath_] = "/";

    treeModel_->append(parent.children());

    cancelBtn_.signal_clicked().connect(
	    sigc::mem_fun(*this, &DirBrowseWnd::OnBtnCancel));
    mainBtnBox_.pack_start(cancelBtn_);

    okBtn_.signal_clicked().connect(
	    sigc::mem_fun(*this, &DirBrowseWnd::OnBtnOk));
    mainBtnBox_.pack_start(okBtn_);

    mainBox_.pack_start(scrolledWnd_);
    mainBox_.pack_start(btnBoxSeparator_, false, false, 5);
    mainBox_.pack_start(mainBtnBox_, false, false);

    show_all_children();
}

DirBrowseWnd::~DirBrowseWnd()
{}

// OK 버튼을 누르면 현재 커서에 해당하는 Path 를 읽어와서 저장
void DirBrowseWnd::OnBtnOk()
{
    Gtk::TreeModel::Path currentPath;
    Gtk::TreeView::Column* currentColumn = NULL;

    treeView_.get_cursor(currentPath, currentColumn);

    Gtk::TreeModel::iterator iter = treeModel_->get_iter(currentPath);
    if(iter) {

	// Gtk::TreeModel::iterator 를 dereference 하면 Gtk::TreeModel::Row 얻음
	selectedDirectory_ = (*iter)[column_.colPath_];
    }

    resultResponse_ = Gtk::RESPONSE_OK;

    hide();
}

void DirBrowseWnd::OnBtnCancel()
{
    resultResponse_ = Gtk::RESPONSE_CANCEL;

    hide();
}

// 처음 보여질 때 현재 디렉토리까지 확장
void DirBrowseWnd::on_show()
{
    Gtk::Window::on_show();

    ExpandCurrentDir();
}

// 해당 row 가 선택되면(더블클릭) 해당 row의 path를 읽어와 저장
void DirBrowseWnd::OnRowActivated(const Gtk::TreeModel::Path& path, Gtk::TreeViewColumn* column)
{

    Gtk::TreeModel::iterator iter = treeModel_->get_iter(path);
    if(iter)
    {
	selectedDirectory_ = (*iter)[column_.colPath_];
    }

    resultResponse_ = Gtk::RESPONSE_OK;

    hide();	
}

// 확장시키는 화살표를 누르면 실제 디렉토리 내용을 읽어와서 완성
void DirBrowseWnd::OnRowExpanded(const Gtk::TreeModel::iterator& iter, const Gtk::TreeModel::Path& path)
{
    // 실제 작업을 하기 전에 커서를 시계로 바꾸고, 표시해줌
    realize();
    get_window()->set_cursor(Gdk::Cursor(Gdk::WATCH));
    while(Glib::MainContext::get_default()->iteration(false));

    // Glib::Dir 대신 직접 DIR 이용
    currentPath_ = (*iter)[column_.colPath_];
    DIR* const currentDir = opendir( currentPath_.c_str());
    dirent* dirEntry;
    vector<string> dirNames; 
    while((dirEntry = readdir(currentDir)) != NULL) {
	fullPath_ = currentPath_;
	fullPath_ += '/'; 
	fullPath_ += dirEntry->d_name;
	if(strncmp(dirEntry->d_name, ".", 1) && strncmp(dirEntry->d_name, "..", 2) 
		&& IsDirectory(fullPath_) ) {
	    dirNames.push_back(dirEntry->d_name);
	}
    }
    closedir(currentDir); 

    bool forDummy;
    // 하위 디렉토리가 있을 경우
    if(!dirNames.empty()) {
	sort(dirNames.begin(), dirNames.end());

	forDummy = true;

	typedef vector<string>::const_iterator veciter;
	veciter vecEnd = dirNames.end();

	Gtk::TreeModel::Row child;
	for(veciter eachFile = dirNames.begin(); eachFile != vecEnd; ++eachFile)
	{
	    fullPath_ = currentPath_;

	    if(fullPath_ != "/") 
		fullPath_ += '/';

	    fullPath_ += *eachFile;

	    // set 에 존재하지 않는지 == 처음 지나가는 디렉토리인지 알아내고 
	    // set에 추가
	    if(!pathInStore_.count(fullPath_)) {

		pathInStore_.insert(fullPath_);

		// dummy 의 경우 내용만 넣어주고, 아닐경우 새로 트리에 추가후
		// 내용을 넣어줌
		if(forDummy) {
		    child = *(iter->children().begin());
		    forDummy = false;
		} else
		    child = *(treeModel_->append(iter->children()));

		// 화면에 보여질 Name 은 UTF8 로 변환
		try {
		    child[column_.colName_] = Glib::locale_to_utf8(*eachFile);
		} catch(Glib::ConvertError& err) {

		    child[column_.colName_] = *eachFile;
		}
		child[column_.colPath_] = fullPath_;

		// 하위 디렉토리 부분을 처리해 줌
		// 일단 접근 가능하고 읽을 수 있다면 dummy 를 추가해서 expander 를
		// 열 수 있도록 함. expander 를 열었을 때 하위 디렉토리가 없다면
		// dummy 를 제거.
		// 주의! 접근 권한이 없는 디렉토리를 접근할 경우 segfault 를 맛볼 수 있음
		if(!eaccess(fullPath_.c_str(), X_OK | R_OK))
		    treeModel_->append(child.children());
	    }
	}

    } else {
	// 하위 디렉토리가 없을 경우 dummy 제거 
	treeModel_->erase(iter->children().begin());
    }	

    // root 윈도우의 커서로 되돌림	
    get_window()->set_cursor();
}

void DirBrowseWnd::ExpandCurrentDir()
{
    vector<string> dirVec;

    if(IsExecutable(selectedDirectory_) && IsReadable(selectedDirectory_))
	SplitEachDir(selectedDirectory_, dirVec);
    else	
	SplitEachDir(Glib::get_current_dir(), dirVec);

    // 루트 부분 열기
    treeView_.expand_row(Gtk::TreePath("0"), false);
    // 루트 바로 밑의 디렉토리들	
    Gtk::TreeNodeChildren child = treeModel_->get_iter("0")->children();
    typedef Gtk::TreeNodeChildren::iterator childiter;
    childiter tempParent;

    Glib::ustring compareName;
    // 루프를 돌면서 현재 디렉토리의 상위 디렉토리에 해당되는 것을 차례로 염
    for(int i = 0; i != dirVec.size(); ++i)
    {
	for(childiter iter = child.begin(); iter != child.end(); ++iter)
	{
	    try {
		compareName = Glib::locale_to_utf8(dirVec[i]);
	    } catch(Glib::ConvertError& err) {
		compareName = dirVec[i];	
	    }

	    if(iter->get_value(column_.colName_) == compareName) {
		tempParent = iter;
		treeView_.expand_row(Gtk::TreePath(iter), false);
	    }
	}
	child = tempParent->children();
    }

    // 현재 디렉토리를 트리뷰 중간에 위치시키고, 선택함
    treeView_.scroll_to_row(Gtk::TreePath(tempParent), 0.5);
    treeView_.set_cursor(Gtk::TreePath(tempParent));

}

