// air is little image thumbnail program.
// Copyright (C) 2004 HG, Rhee <ez8tank@krpost.net>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
//
// All comments are korean, UTF-8 encoding.
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "../include/mainwin.h"
#include "../include/prefwindow.h"
#include "../include/singlescalewin.h"
#include "../include/morescalewin.h"
#include "../include/dirbrowsewin.h"
#include "../include/misc.h"

using std::list;
using std::string;

MainWnd::MainWnd()
{
    set_title("air");
    set_size_request(200, 320);
    set_resizable(false);

    add(mainBox_);

    fileMenu_.items().push_back( Gtk::Menu_Helpers::MenuElem("화일 처리하기",
        sigc::mem_fun(*this, &MainWnd::OnMenuFileopen)) );
    fileMenu_.items().push_back( Gtk::Menu_Helpers::MenuElem("폴더 처리하기",
	sigc::mem_fun(*this, &MainWnd::OnMenuFolderopen)) );
    fileMenu_.items().push_back( Gtk::Menu_Helpers::SeparatorElem());
    fileMenu_.items().push_back( Gtk::Menu_Helpers::MenuElem("종료",
        sigc::mem_fun(*this, &MainWnd::OnMenuQuit)) );
    menuBar_.items().push_back( Gtk::Menu_Helpers::MenuElem("화일", fileMenu_) );
 
    editMenu_.items().push_back( Gtk::Menu_Helpers::MenuElem("설정",
	sigc::mem_fun(*this, &MainWnd::OnMenuPreferences)) );	
    menuBar_.items().push_back( Gtk::Menu_Helpers::MenuElem("편집", editMenu_) );
	
    aboutMenu_.items().push_back( Gtk::Menu_Helpers::MenuElem("정보",
	sigc::mem_fun(*this, &MainWnd::OnMenuAbout)) );
    menuBar_.items().push_back( Gtk::Menu_Helpers::MenuElem("정보", aboutMenu_) );	
    
    mainBox_.pack_start(menuBar_, Gtk::PACK_SHRINK);

    show_all_children();
}

MainWnd::~MainWnd()
{}

void MainWnd::OnMenuFileopen()
{
    Gtk::FileSelection fileDlg("화일을 선택하세요");
    fileDlg.set_transient_for(*this);
    fileDlg.set_select_multiple(true);
    fileDlg.hide_fileop_buttons();

    int result = fileDlg.run();
    fileDlg.hide();

    switch(result)
    {
	case(Gtk::RESPONSE_OK):
	    {
		// 화일 선택 다이얼로그에서 선택된 것을 읽어와서
		// 해당 확장자외에는 지움
		selectedFiles_ = fileDlg.get_selections();
		if(preference_.ExtFilter()) {
		    selectedFiles_.remove_if(std::bind1st(NotInFilterExt(), preference_.FilterExt()));
		}

		if(selectedFiles_.size() == 1 && !IsDirectory(*(selectedFiles_.begin()))) {

		    Glib::RefPtr<Gdk::Pixbuf> image;
		    try {
			image = Gdk::Pixbuf::create_from_file(*(selectedFiles_.begin()));

		    } catch(Gdk::PixbufError &err) {
			Gtk::MessageDialog errDlg(*this, err.what(), false, Gtk::MESSAGE_ERROR);
			errDlg.run();	
		    }

		    if(image) {
			SingleScaleWnd singleScaleWnd(preference_, *(selectedFiles_.begin()), image);
			singleScaleWnd.set_modal(true);
			Gtk::Main::run(singleScaleWnd);
		    }

		} else {
		    ChooseWnd chooseWnd(preference_, selectedFiles_);
		    chooseWnd.set_modal(true);
		    Gtk::Main::run(chooseWnd);
		}

		break;
	    }
	case(Gtk::RESPONSE_CANCEL):
	    {
		break;
	    }
    }

}

void MainWnd::OnMenuFolderopen()
{
    DirBrowseWnd dirBrowseWnd;
    dirBrowseWnd.set_modal(true);
    Gtk::Main::run(dirBrowseWnd);

    if(dirBrowseWnd.ResultResponse() == Gtk::RESPONSE_OK && 
	!dirBrowseWnd.SelectedDirectory().empty()) {
	
	list<string>().swap(selectedFiles_);
	GetFileList(dirBrowseWnd.SelectedDirectory(), selectedFiles_, preference_.Recursive());
	if(preference_.ExtFilter()) {
	    selectedFiles_.remove_if(std::bind1st(NotInFilterExt(), preference_.FilterExt()));
	}

	if(!selectedFiles_.empty()) {
	    ChooseWnd chooseWnd(preference_, selectedFiles_);
	    chooseWnd.set_modal(true);
	    Gtk::Main::run(chooseWnd);
	}
    }	
}

void MainWnd::OnMenuPreferences()
{
    PrefWnd prefWnd(preference_);
    prefWnd.set_modal(true);
    Gtk::Main::run(prefWnd);
}

void MainWnd::OnMenuAbout()
{	
    // 그다지 열리지 않을 것을 예상하여 문자열을 미리 만들어 두지 않음
    Glib::ustring aboutString = "<span weight=\"bold\" size=\"xx-large\">air v" PACKAGE_VERSION "</span>\n\n간단한 이미지 줄이개\n\n";		
    AddFormatInfo(aboutString);
    aboutString += "\n\n<span size=\"small\">Copyright (c) 2004 Ez8(ez8tank@krpost.net) under GNU GPL2</span>";

    Gtk::MessageDialog aboutDlg(*this, aboutString, true);
    aboutDlg.set_border_width(6);
    aboutDlg.get_vbox()->set_border_width(12);
    aboutDlg.run();
}

void MainWnd::OnMenuQuit()
{
    hide();
}

void MainWnd::AddFormatInfo(Glib::ustring& orig)
{
    orig += "읽을 수 있는 형식 : ";

    for_each(preference_.ReadableFormats().begin(), preference_.ReadableFormats().end(),
	    bind1st( GetFormatName(), &orig));

    orig += "\n\n쓸 수 있는 형식 : ";

    for_each(preference_.WritableFormats().begin(), preference_.WritableFormats().end(),
	    bind1st( GetFormatName(), &orig));

}
