// air is little image thumbnail program.
// Copyright (C) 2004 HG, Rhee <ez8tank@krpost.net>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
//
// All comments are korean, UTF-8 encoding.

#include <iostream>
#include <fstream>
#include <algorithm>

#include "../include/misc.h"

using std::ifstream;
using std::vector;
using std::list;
using std::map;
using std::string;
using std::isspace;
using std::isdigit;

// 절대 경로 명에서 경로, 화일명, 확장자를 분리시킨 컨테이너 반환
void SplitPath(const string& fullpath, vector<string>& dest)
{
    string::const_iterator begin, end, path, ext;

    begin = fullpath.begin();
    end = fullpath.end();

    for(string::const_iterator i = begin; i != end; ++i) {
	// 계속 증가 시키며 대입되므로 마지막 부분에 걸림
	if(*i == '/') 
	    path = i;
	if(*i == '.') 
	    ext = i;	
    }

    // 처음에서 path 까지 자르면 마지막 /가 포함이 안됨	
    path++;

    dest.push_back(string(begin, path));
    dest.push_back(string(path, ext));
    if(ext != end && ext > path) 
	dest.push_back(string(ext, end));
}

void SplitEachDir(const string& fullpath, vector<string>& dest)
{
    typedef string::const_iterator iter;

    iter begin = fullpath.begin();
    iter end = fullpath.end();
    iter temp;
    while(begin != end)
    {
	begin = find_if(begin, end, std::not1(Slash()));
	temp = find_if(begin, end, Slash());
	if(begin != end)
	    dest.push_back(string(begin, temp));
	begin = temp;		
    }

}

// 콤마를 기준으로 뒷부분을 분리시킨 문자열(확장자 제거) 반환(패스 고려 안함)
Glib::ustring StripExt(const Glib::ustring& p)
{
    typedef Glib::ustring::const_iterator iter;

    iter begin = p.begin();
    iter end = p.end();
    iter coma = end;

    for(iter i = begin; i != end; ++i)
	if(*i == '.') 
	    coma = i;

    // 콤마를 발견 못하면 콤마가 end 이므로 전체 복사가 됨		
    return Glib::ustring(begin, coma);	
}

// 콤마를 기준으로 앞부분을 분리시킨 문자열(화일 이름 제거) 반환(패스 고려)
string StripName(const string& p)
{
    typedef string::const_iterator iter;

    iter path, ext;	
    iter end = p.end();

    for(iter i = p.begin(); i != end; ++i) {
	if(*i == '/') path = i;
	if(*i == '.') ext = i;		
    }

    if(ext != end && ext > path) 
	return string(ext + 1, end);
    else 
	return string();
}

// 특정 절대 경로 밑의 (재귀적인) 화일 리스트의 컨테이너를 반환.
void GetFileList(const std::string& path, std::list<std::string>& dest, bool recursive)
{
    Glib::Dir dir(path);

    string fullpath;
    for(Glib::Dir::const_iterator eachFile = dir.begin(); eachFile != dir.end(); ++eachFile)
    {
	fullpath = path;
	fullpath += '/';
	fullpath += *eachFile;
	
	if(IsDirectory(fullpath)) {
	    if(recursive && IsExecutable(fullpath) && IsReadable(fullpath)) {
		GetFileList(fullpath, dest, recursive);
	    }
	} else {
	    dest.push_back(fullpath);
	}
    }

}

PreviewArea::PreviewArea()
: imageWidth_(0),
    imageHeight_(0)
{}

PreviewArea::~PreviewArea()
{}

bool PreviewArea::on_expose_event(GdkEventExpose*)
{
    if(image_)
	Draw();

    return true;
}

void PreviewArea::SetImg(Glib::RefPtr<Gdk::Pixbuf> img)
{
    imageWidth_ = img->get_width();
    imageHeight_ = img->get_height();

    float ratio;
    if(imageWidth_ > imageHeight_) {
	ratio = 100 / static_cast<float>(imageWidth_);
    } else {
	ratio = 100 / static_cast<float>(imageHeight_);
    }

    imageWidth_ = static_cast<int>(ratio * imageWidth_);
    imageHeight_ = static_cast<int>(ratio * imageHeight_);

    image_ = img->scale_simple(imageWidth_, imageHeight_, Gdk::INTERP_NEAREST);
}

void PreviewArea::Draw()
{
    if(image_) 
    {
	if(!is_realized()) 
	    realize();

	get_window()->clear_area(get_width()/2 - 50, 0, 100, 100);
	image_->render_to_drawable(get_window(), get_style()->get_black_gc(),
		0, 0, static_cast<int>(get_width()/2) - static_cast<int>(imageWidth_/2), 50 - static_cast<int>(imageHeight_/2), 
		imageWidth_, imageHeight_,
		Gdk::RGB_DITHER_NONE, 0, 0);
    }
}

// 홈 디렉토리 밑에 설정화일에서 화일을 읽어와서 등호를 기준으로
// 값을 분리시킨 연관 컨테이너를 작성함
bool LoadConfig(map<string, string>& p)
{
    string configfilePath = Glib::get_home_dir();
    configfilePath += "/.air";

    ifstream configfile(configfilePath.c_str());
    string buf;

    if(!configfile) 
	return false;
    else 
	while(getline(configfile, buf)) {
	    ProcConfigLine thisline(buf);
	    if(thisline.valid()) {
		p[thisline.key()] = thisline.value();
	    }
	}
    return true;
}

// 설정화일에서 각 라인를 처리함
// 등호를 기준으로 앞쪽이 키, 뒷쪽이 값이 되는 적절한 
// 경우를 찾아 문자열로 저장.
ProcConfigLine::ProcConfigLine(string& p)
: valid_(false)
{
    typedef string::iterator iter;

    // 모두 소문자로 바꿈
    for_each(p.begin(), p.end(), ToLower());

    // 유효한 패턴
    // 1. foo = bar
    // 2. foo= bar
    // 3. foo =bar
    // 4. foo=bar

    // key 부분을 찾음
    // 앞쪽 공백 제거
    iter i = p.begin();
    i = find_if(i, p.end(), std::not1(Space()));		
    // 등호 문자 검색
    iter j = find_if(i, p.end(), Equal());
    if(i != p.end() && i != j) {
	// [i, j) 의 공백을 제거하여 key에 넣음
	iter k = find_if(i, j, Space());
	key_ = string(i, k);

	// value 부분을 찾음
	i = ++j;
	i = find_if(i, p.end(), std::not1(Space()));
	j = find_if(i, p.end(), Space());
	if(i != p.end()) {
	    value_ = string(i, j);
	    valid_ = true;
	}
    }		
}
