// air is little image thumbnail program.
// Copyright (C) 2004 HG, Rhee <ez8tank@krpost.net>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
//
// All comments are korean, UTF-8 encoding.

#include <vector>
#include <iostream>
#include <sstream>

#include "../include/singlescalewin.h"
#include "../include/morescalewin.h"

using std::vector;
using std::list;
using std::string;
using std::stringstream;

ChooseWnd::ChooseWnd(const Preference& preference, list<string> filenames)
    : preference_(preference),
    filenames_(filenames),
    mainTable_(3, 12),
    mainBtnBox_(Gtk::BUTTONBOX_END, 12),
    infoIcon_(Gtk::Stock::DIALOG_INFO,Gtk::ICON_SIZE_LARGE_TOOLBAR),
    infoHeadLabel_("<span weight=\"bold\">정보</span>", 0.0, 0.5),
    batchHeadLabel_("<span weight=\"bold\">일괄처리 설정</span>", 0.0, 0.5),
    eachProc_("개별적으로 처리"),
    batchProc_("일괄적으로 처리"),
    cancelBtn_(Gtk::Stock::CANCEL),
    okBtn_(Gtk::Stock::OK),
    widthLabel_("새로운 가로 : "),
    heightLabel_("새로운 세로 : "),
    typeLabel_("저장할 형식 : "),
    widthAdj_(1.0, 1.0, 10000.0, 1.0, 5.0, 0.0),
    heightAdj_(1.0, 1.0, 10000.0, 1.0, 5.0, 0.0)
{
    set_title("선택 창");
    set_border_width(12);
    set_resizable(false);
    set_modal(true);

    mainTable_.set_row_spacings(6);
    mainTable_.set_col_spacing(0, 12);
    mainTable_.set_col_spacing(1, 12);
    add(mainTable_);

    // /---------------------\
    // | label               |
    // | | hbox              |
    // | | radio btn         |
    // | | radio btn         |
    // |---------------------|
    // | label               |
    // | | label | spin btn  |
    // | | label | spin btn  |
    // | | label | hbox      |
    // |---------------------|
    // | hseparator          |
    // | hBtnBox             |
    // \---------------------/

    infoHeadLabel_.set_use_markup(true);
    mainTable_.attach(infoHeadLabel_, 0, 3, 0, 1, Gtk::FILL);

    stringstream buf;
    buf << "<span style=\"italic\"> " << filenames_.size() << "개의 화일이 선택되었습니다.</span>";
    infoLabel_.set_text(buf.str());
    infoLabel_.set_use_markup(true);
    infoBox_.pack_start(infoIcon_, Gtk::PACK_SHRINK);
    infoBox_.pack_start(infoLabel_, Gtk::PACK_SHRINK);
    mainTable_.attach(infoBox_, 1, 3, 1, 2, Gtk::FILL);

    Gtk::RadioButton::Group group = eachProc_.get_group();
    eachProc_.signal_clicked().connect(
	    sigc::mem_fun(*this, &ChooseWnd::OnRadioBatch));
    batchProc_.set_group(group);
    batchProc_.set_active(true);
    batchProc_.signal_clicked().connect(
	    sigc::mem_fun(*this, &ChooseWnd::OnRadioBatch));
    mainTable_.attach(eachProc_, 1, 3, 2, 3, Gtk::FILL);
    mainTable_.attach(batchProc_, 1, 3, 3, 4, Gtk::FILL);

    mainTable_.attach(dummyLabel1, 0, 1, 1, 4, Gtk::SHRINK);
    mainTable_.attach(dummyLabel2, 0, 3, 4, 5, Gtk::SHRINK);

    batchHeadLabel_.set_use_markup(true);
    mainTable_.attach(batchHeadLabel_, 0, 3, 5, 6, Gtk::FILL);

    widthSpin_.set_adjustment(widthAdj_);
    widthSpin_.set_value(100);
    mainTable_.attach(widthLabel_, 1, 2, 6, 7, Gtk::FILL);
    mainTable_.attach(widthSpin_, 2, 3, 6, 7, Gtk::FILL);

    heightSpin_.set_adjustment(heightAdj_);
    heightSpin_.set_value(100);
    mainTable_.attach(heightLabel_, 1, 2, 7, 8, Gtk::FILL);
    mainTable_.attach(heightSpin_, 2, 3, 7, 8, Gtk::FILL);

    Gtk::RadioButton::Group extGroup;
    for(vector<Gdk::PixbufFormat>::const_iterator i = preference_.WritableFormats().begin(); 
	    i != preference_.WritableFormats().end(); ++i) {
	if(ext_.empty()) ext_ = i->get_name();
	Gtk::RadioButton* rb = manage(new Gtk::RadioButton(extGroup, i->get_name()));
	rb->signal_clicked().connect( sigc::bind(
		    sigc::mem_fun(*this, &ChooseWnd::OnRadioExt), i->get_name() ) );
	extRadioBox_.pack_start(*rb);
    }	
    mainTable_.attach(typeLabel_, 1, 2, 8, 9, Gtk::FILL);
    mainTable_.attach(extRadioBox_, 2, 3, 8, 9, Gtk::FILL);

    mainTable_.attach(dummyLabel3, 0, 1, 6, 9, Gtk::SHRINK);
    mainTable_.attach(dummyLabel4, 0, 3, 9, 10, Gtk::SHRINK);

    mainTable_.attach(btnBoxSeparator_, 0, 3, 10, 11, Gtk::FILL);

    cancelBtn_.signal_clicked().connect(
	    sigc::mem_fun(*this, &ChooseWnd::OnBtnCancel));
    mainBtnBox_.pack_start(cancelBtn_);
    okBtn_.signal_clicked().connect(
	    sigc::mem_fun(*this, &ChooseWnd::OnBtnOk));
    mainBtnBox_.pack_start(okBtn_);
    
    mainTable_.attach(mainBtnBox_, 0, 3, 11, 12, Gtk::FILL);	

    show_all_children();
}

ChooseWnd::~ChooseWnd()
{
}

void ChooseWnd::OnRadioBatch()
{
    if(batchProc_.get_active()) {
	widthSpin_.set_sensitive(true);
	heightSpin_.set_sensitive(true);
	extRadioBox_.set_sensitive(true);        
    } else {
	widthSpin_.set_sensitive(false);
	heightSpin_.set_sensitive(false);
	extRadioBox_.set_sensitive(false); 
    }
}

void ChooseWnd::OnRadioExt(const string& p)
{
    ext_ = p;
}

void ChooseWnd::OnBtnCancel()
{
    hide();
}

void ChooseWnd::OnBtnOk()
{
    if(eachProc_.get_active()) {
	typedef list<string>::const_iterator iter;
	
	Glib::RefPtr<Gdk::Pixbuf> image;
	for(iter i = filenames_.begin(); i != filenames_.end(); i++) {
	    if(!IsDirectory(*i)) {
		try {
		    image = Gdk::Pixbuf::create_from_file(*i);					
		} catch(Gdk::PixbufError &err) {
		    Gtk::MessageDialog errDlg(*this, err.what(), false, Gtk::MESSAGE_ERROR);
		    errDlg.run();	
		} if(image) {
		    hide();
		    SingleScaleWnd singleScaleWnd(preference_, *i, image);
		    Gtk::Main::run(singleScaleWnd);
		}
	    }
	}
    } else if(batchProc_.get_active()) {
	hide();
	MoreScaleWnd moreScaleWnd(preference_, 
		widthSpin_.get_value_as_int(), 
		heightSpin_.get_value_as_int(), 
		filenames_, ext_);
	Gtk::Main::run(moreScaleWnd);	

    }

}

MoreScaleWnd::MoreScaleWnd
(const Preference& preference, int width, int height, list<string> filenames, string ext)
    : preference_(preference),
    width_(width),
    height_(height),
    filenames_(filenames),
    ext_(ext),
    inPause_(true),
    breakThread_(false),
    endThread_(false),
    subLabel_("", 0.0, 0.5, false),
    pauseBtn_("일시정지"),
    previewArea_(NULL),
    totalFileNum_(filenames_.size()),
    fileCount_(0),
    execThread_(NULL),
    mainTable_(1,6),
    infoHeadLabel_("<span weight=\"bold\" size=\"x-large\">이미지 변환 중</span>", 0.0, 0.5),
    mainBtnBox_(Gtk::BUTTONBOX_END, 12),
    cancelBtn_(Gtk::Stock::CANCEL),
    progressBarText_(""),
    progressBarFraction_(0.0),
    subLabelText_(""),
    image_(NULL),
    errDlgResponse(0),
    errDlgGetResponse(false)
{
    set_title("이미지 변환 중");
    set_resizable(false);
    set_border_width(12);
    set_modal(true);

    mainTable_.set_row_spacings(6);
    add(mainTable_);

    infoHeadLabel_.set_use_markup(true);
    mainTable_.attach(infoHeadLabel_, 0, 1, 0, 1, Gtk::FILL);

    if(preference_.Preview()) {
	previewArea_ = manage(new PreviewArea);
	previewArea_->set_size_request(100, 100);
	mainTable_.attach(*previewArea_, 0, 1, 1, 2, Gtk::FILL);
    } else {
	mainTable_.attach(*manage(new Gtk::Label), 0, 1, 1, 2, Gtk::SHRINK);
    }

    stringstream buf;
    buf << totalFileNum_ << "개의 화일을 " << width_ << " x " << height_ << " 로 변환 중 입니다.";
    infoLabel_.set_text(buf.str());
    mainTable_.attach(infoLabel_, 0, 1, 2, 3, Gtk::FILL);

    progressBar_.set_size_request(-1, 27);
    progressBox_.pack_start(progressBar_, Gtk::PACK_SHRINK);
    progressBox_.pack_start(subLabel_, Gtk::PACK_SHRINK);
    mainTable_.attach(progressBox_, 0, 1, 3, 4, Gtk::FILL);
    mainTable_.attach(dummyLabel_, 0, 1, 4, 5, Gtk::FILL);

    cancelBtn_.signal_clicked().connect(
	    sigc::mem_fun(*this, &MoreScaleWnd::OnBtnCancel) );
    mainBtnBox_.pack_start(cancelBtn_);

    pauseBtn_.signal_clicked().connect(
	    sigc::mem_fun(*this, &MoreScaleWnd::OnBtnPause) );
    mainBtnBox_.pack_start(pauseBtn_);


    mainTable_.attach(mainBtnBox_, 0, 1, 5, 6, Gtk::FILL);

    this->signal_show().connect(
	    sigc::mem_fun(*this, &MoreScaleWnd::OnShow));

    show_all_children();
}

MoreScaleWnd::~MoreScaleWnd()
{
}

void MoreScaleWnd::OnBtnPause()
{
    if(endThread_) {
	hide();
    }

    if(inPause_) {
	inPause_ = false;
	pauseBtn_.set_label("일시정지");	
    } else {
	inPause_ = true;
	pauseBtn_.set_label("계속");
    }
}

void MoreScaleWnd::OnShow()
{
    inPause_ = false;
    
    // 실행 thread 에서 GUI 부분의 처리을 요청할 시그널 연결
    signalSetProgressBarText_.connect(sigc::mem_fun(*this, &MoreScaleWnd::SetProgressBarText));
    signalSetProgressBarFraction_.connect(sigc::mem_fun(*this, &MoreScaleWnd::SetProgressBarFraction));
    signalSetSubLabelText_.connect(sigc::mem_fun(*this, &MoreScaleWnd::SetSubLabel));
    signalUpdatePreview_.connect(sigc::mem_fun(*this, &MoreScaleWnd::UpdatePreview));
    signalFileExistsErr_.connect(sigc::mem_fun(*this, &MoreScaleWnd::FileExistsErrDlg));
    signalExecFinish_.connect(sigc::mem_fun(*this, &MoreScaleWnd::ExecFinish));
    
    // 실제 수행 thread 를 만듬
    execThread_ = Glib::Thread::create(sigc::mem_fun(*this, &MoreScaleWnd::Exec), false);
}

void MoreScaleWnd::SetProgressBarText()
{
    progressBar_.set_text("");
    progressBar_.set_text(progressBarText_);
}

void MoreScaleWnd::SetProgressBarFraction()
{
    progressBar_.set_fraction(progressBarFraction_);
}

void MoreScaleWnd::SetSubLabel()
{
    subLabel_.set_text("<span style=\"italic\">" + subLabelText_ + "</span>");
    subLabel_.set_use_markup(true);
}

void MoreScaleWnd::UpdatePreview()
{
    previewArea_->SetImg(image_);
    previewArea_->Draw();

}    

void MoreScaleWnd::FileExistsErrDlg()
{
    Gtk::MessageDialog errDlg(*this, "화일이 이미 존재합니다.\n 덮어씁니까?", false, Gtk::MESSAGE_QUESTION,
	    Gtk::BUTTONS_OK_CANCEL );    
    errDlgResponse = errDlg.run(); 
    errDlgGetResponse = true;
}

void MoreScaleWnd::ExecFinish()
{
    pauseBtn_.set_label("닫기");
    subLabel_.set_text(""); 
}

void MoreScaleWnd::Exec()
{
    bool skipThisFile = false;
    bool overWriteFile = preference_.FileaddType() == 2;
    vector<string> origFilePath;
    string saveFilename;
    string origFilename;

    // 넘겨 받은 컨테이너에는 디렉토리가 포함되어 있지 않음
    list<string>::const_iterator eachFile = filenames_.begin();
    for(;;) 
    {
	if(!inPause_) {
	    if(eachFile != filenames_.end()) {
		fileCount_++;

		// 컨테이너를 비우고 다시 받음
		vector<string>().swap(origFilePath);
		SplitPath(*eachFile, origFilePath);

		saveFilename = origFilePath[0];
		saveFilename += preference_.ConvertFilename(origFilePath[1]);

		if(ext_ == "jpeg")
		    saveFilename += ".jpg";
		else {
		    saveFilename += ".";
		    saveFilename += ext_;
		}

		stringstream buf;
		buf << totalFileNum_ << " 개 중 " << fileCount_ << " 개의 화일을 처리했습니다";

		progressBarText_ = buf.str();
		signalSetProgressBarText_();

		progressBarFraction_ = fileCount_/static_cast<double>(totalFileNum_);
		signalSetProgressBarFraction_();

		origFilename = "\"";
		origFilename += origFilePath[1];
		if(origFilePath.size() == 3)
		    origFilename += origFilePath[2];
		Glib::ustring temp = origFilename;
		try {
		    origFilename = Glib::locale_to_utf8(temp);
		} catch(Glib::ConvertError& err) {
		    origFilename = temp;
		}

		origFilename += "\" 을 변환 중 입니다";
		subLabelText_ = origFilename;
		signalSetSubLabelText_();

		try {
		    image_ = Gdk::Pixbuf::create_from_file(*eachFile);
		} catch(Gdk::PixbufError &err) {
		    if(preference_.StopError())
			OnBtnPause();

		    skipThisFile = true;

		    subLabelText_ = err.what();
		    signalSetSubLabelText_();
		}

		if(!skipThisFile && image_) {
		    if(preference_.Preview()) {
			signalUpdatePreview_();
		    }

		    if(!overWriteFile && Glib::file_test(saveFilename, Glib::FILE_TEST_EXISTS)) {
			if(preference_.StopError()) {
			    signalFileExistsErr_();

			    while(!errDlgGetResponse) {

				Glib::usleep(10000);   
			    }

			    switch(errDlgResponse)
			    {
				case(Gtk::RESPONSE_OK):
				    {
					SaveFile(image_, saveFilename);
					break;
				    }
				case(Gtk::RESPONSE_CANCEL):
				    {
					break;
				    }
			    }
			} else {
			    subLabelText_ = "\"";
			    try {
				subLabelText_ += Glib::locale_to_utf8(saveFilename);
			    } catch(Glib::ConvertError& err) {
				subLabelText_ += saveFilename;
			    }
			    subLabelText_ += "\" 화일이 이미 존재 합니다";

			    signalSetSubLabelText_();
			}

		    } else {
			SaveFile(image_, saveFilename);
		    }
		}

	    }	
	    skipThisFile = false;
	    ++eachFile;
	}

	if(fileCount_ == totalFileNum_) { 
	    signalExecFinish_();
	    break;
	}

	if(breakThread_) {
	    break;
	}    

	Glib::usleep(10000);
    }

    endThread_ = true;

}

bool MoreScaleWnd::on_delete_event(GdkEventAny* ev)
{
    OnBtnCancel();
}

void MoreScaleWnd::OnBtnCancel()
{
    breakThread_ = true;
    
    while(!endThread_) {
	Glib::usleep(10000);
    }

    hide();
}

void MoreScaleWnd::SaveFile(Glib::RefPtr<Gdk::Pixbuf> origImage, string& filename)
{

    Glib::RefPtr<Gdk::Pixbuf> dest = 
	origImage->scale_simple(width_, height_, preference_.InterpType());

    try {
	if(ext_ == "jpeg") {
	    dest->save(filename, "jpeg", preference_.JpegOptionKeys(), preference_.JpegOptionValues());
	}
	else dest->save(filename, ext_);

    } catch(Gdk::PixbufError& err) {
	if(preference_.StopError()) {
	    OnBtnPause();
	}
	subLabelText_ = err.what();
	signalSetSubLabelText_();

    } catch(Glib::FileError& err) {
	if(preference_.StopError()) {
	    OnBtnPause();
	}
	subLabelText_ = err.what();
	signalSetSubLabelText_();
    }

}

