// air is little image thumbnail program.
// Copyright (C) 2004 HG, Rhee <ez8tank@krpost.net>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
//
// All comments are korean, UTF-8 encoding.

#include "../include/preference.h"
#include "../include/misc.h"

using std::vector;
using std::map;
using std::set;
using std::string;
using std::stringstream;

Preference::Preference()
    : fileaddString_("tn_"),
    jpegcomp_(75),
    interpType_(Gdk::INTERP_BILINEAR),
    fileaddType_(FILEADD_PREFIX),
    stopError_(true),
    recursive_(false),
    extFilter_(true),
    preview_(false)
{
    readableFormats_.reserve(15);
    readableFormats_ = Gdk::Pixbuf::get_formats();	
    vector<Gdk::PixbufFormat>(readableFormats_).swap(readableFormats_);

    // 기본적으로 png, jpeg, ico 가 쓸 수 있는 포맷.
    writableFormats_.reserve(3);
    for_each(readableFormats_.begin(), readableFormats_.end(), 
	    std::bind1st(SeparateWritableFormats(), &writableFormats_));	
    vector<Gdk::PixbufFormat>(writableFormats_).swap(writableFormats_);

    for_each(readableFormats_.begin(), readableFormats_.end(), std::bind1st(InsertStringSet(), &filterExt_));

    jpegOptionKeys_.push_back("quality");
    stringstream buf;
    buf << jpegcomp_;
    jpegOptionValues_.push_back(buf.str());

    map<string, string> configfile;
    // 각 변수에 config의 해당 키 값을 검사해서 대입
    if(LoadConfig(configfile)) {
	AssignConfig(preview_, configfile, "preview", IsNumBoolean);
	AssignConfig(stopError_, configfile, "stoperror", IsNumBoolean);
	AssignConfig(recursive_, configfile, "recursive", IsNumBoolean);
	AssignConfig(extFilter_, configfile, "extfilter", IsNumBoolean);
	AssignConfig(interpType_, configfile, "interptype", IsInteger);
	AssignConfig(jpegcomp_, configfile, "jpegcomp", IsInteger);
	AssignConfig(fileaddType_, configfile, "fileaddtype", IsInteger);
	AssignConfig(fileaddString_, configfile, "fileaddstring");
    }
}

// 설정에서 정한 대로 화일 이름 부분을 조작해서 반환
void Preference::ConvertFilename(const Glib::ustring& filename, Glib::ustring& dest) const
{
    switch(fileaddType_) {
	case FILEADD_PREFIX:
	    dest = fileaddString_;
	    dest += filename;
	    break;
	    
	case FILEADD_SUFFIX:
	    dest = filename;
	    dest += fileaddString_;
	    break;
	    
	case FILEADD_OVERWRITE:
	    dest = filename;
	    break;        

    }
}

Glib::ustring Preference::ConvertFilename(const Glib::ustring& filename) const
{
    switch(fileaddType_) {
	case FILEADD_PREFIX:
	    return Glib::ustring(fileaddString_ + filename);

	case FILEADD_SUFFIX:
	    return Glib::ustring(filename + fileaddString_);

	case FILEADD_OVERWRITE:
	    return filename;

    }
}

    template <typename T>
void Preference::AssignConfig(T& p, const map<string, string>& configfile, const string& str, bool (*f)(const string&))
{
    map<string, string>::const_iterator i = configfile.find(str);
    if( i != configfile.end() && f(i->second)) 
	p = atoi(i->second.c_str());
}

void Preference::AssignConfig(Gdk::InterpType& p, const map<string, string>& configfile, const string& str, bool (*f)(const string&))
{
    map<string, string>::const_iterator i = configfile.find(str);
    if( i != configfile.end() && f(i->second)) {
	int buf = atoi(i->second.c_str());
	if( buf >= 0 && buf <= 	3) 
	    switch(buf) {
		case 0:
		    p = Gdk::INTERP_NEAREST;
		    break;
		case 1:
		    p = Gdk::INTERP_TILES;
		    break;
		case 2:
		    p = Gdk::INTERP_BILINEAR;
		    break;
		case 3:
		    p = Gdk::INTERP_HYPER;
		    break;	
	    }
    }
}

void Preference::AssignConfig(FileAddType& p, const map<string, string>& configfile, const string& str, bool (*f)(const string&))
{
    map<string, string>::const_iterator i = configfile.find(str);
    if( i != configfile.end() && f(i->second)) {
	int buf = atoi(i->second.c_str());
	if( buf >=0 && buf <= 2) 
	    switch(buf) {
		case 0:
		    p = FILEADD_PREFIX;
		    break;
		case 1:
		    p = FILEADD_SUFFIX;
		    break;
		case 2:
		    p = FILEADD_OVERWRITE;
		    break;
	    }	
    }
}

void Preference::AssignConfig(string& p, const map<string, string>& configfile, const string& str)
{
    map<string, string>::const_iterator i = configfile.find(str);
    if( i != configfile.end() ) 
	p = i->second;
}

