// air is little image thumbnail program.
// Copyright (C) 2004 HG, Rhee <ez8tank@krpost.net>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
//
// All comments are korean, UTF-8 encoding.

#include <fstream>
#include <cstdlib>
#include "../include/prefwindow.h"
#include "../include/misc.h"

using std::ostream;
using std::ofstream;
using std::endl;
using std::string;
using std::vector;
using std::map;

PrefWnd::PrefWnd(Preference& preference)
    : mainTable_(3, 14),
    mainBtnBox_(Gtk::BUTTONBOX_END, 12),
    fileaddRadioBox_(false, 5),
    baseHeadLabel_("<span weight=\"bold\">기본</span>", 0.0, 0.5),
    baseInterpLabel_("보간법 : ", 0.0, 0.5),
    baseJpegcompLabel_("jpeg 품질 : ", 0.0, 0.5),
    batchHeadLabel_("<span weight=\"bold\">일괄처리</span>", 0.0, 0.5),
    batchFileaddLabel_("화일에 덧붙일 이름 : ", 0.0, 0.5),
    prefixBtn_("화일명 앞에"),
    suffixBtn_("화일명 뒤에"),
    overwriteBtn_("덮어쓰기"),
    previewBtn_("미리보기"),
    stoperrorBtn_("에러 발생 시 정지"),
    recursiveBtn_("하위 디렉토리까지 재귀적으로 처리"),
    extFilterBtn_("확장자로 미리 거르기"),
    saveBtn_(Gtk::Stock::SAVE),
    cancelBtn_(Gtk::Stock::CANCEL),
    applyBtn_(Gtk::Stock::APPLY),
    jpegcompScale_(0.0, 100.0, 1.0),
    preference_(preference)
{
    set_title("설정 창");
    set_border_width(12);
    set_resizable(false);

    // 함수 이름으로만 생각하면 실제로 사용시 혼동되기 쉬움
    // set_row_spacings - 가로 줄에 공통으로 적용 될 spacing
    // set_col_spacings - 세로 줄에 공통으로 적용 될 spacing
    // set_row_spacing  - 특정 가로 줄에 적용될 spacing
    //		    - parameter가 table 생성시의 col 범위
    // set_col_spacing  - 특정 세로 줄에 적용될 spacing
    //		    - parameter가 table 생성시의 row 범위
    mainTable_.set_row_spacings(6);
    mainTable_.set_col_spacing(0, 12);
    mainTable_.set_col_spacing(1, 12);
    add(mainTable_);

    //  /-------------------------\ 
    //  | label                   |
    //  | | chk btn               |
    //  | | chk btn               |
    //  | | label | option        |
    //  | | label | slider        |
    //  |-------------------------|
    //  | label                   |
    //  | | chkbtn                |
    //  | | chkbtn                |
    //  | | label | radio         |
    //  | |       | entry         |
    //  |-------------------------|
    //  | hseparator              |
    //  | hbtnbox                 |
    //  \-------------------------/
    baseHeadLabel_.set_use_markup(true);
    mainTable_.attach(baseHeadLabel_, 0, 3, 0, 1, Gtk::FILL);	

    // 미리보기, 확장자 필터 체크 버튼
    previewBtn_.set_active(preference_.Preview());
    mainTable_.attach(previewBtn_, 1, 3, 1, 2, Gtk::FILL);

    extFilterBtn_.set_active(preference_.ExtFilter());
    mainTable_.attach(extFilterBtn_, 1, 3, 2, 3, Gtk::FILL);

    // 보간법 설정
    mainTable_.attach(baseInterpLabel_, 1, 2, 3, 4, Gtk::FILL);
    Gtk::Menu_Helpers::MenuList& list_vpos = interpMenu_.items();
    list_vpos.push_back( Gtk::Menu_Helpers::MenuElem("근접", 
		sigc::bind( sigc::mem_fun(*this, &PrefWnd::OnMenuInterp), Gdk::INTERP_NEAREST) ) );
    list_vpos.push_back( Gtk::Menu_Helpers::MenuElem("타일", 
		sigc::bind( sigc::mem_fun(*this, &PrefWnd::OnMenuInterp), Gdk::INTERP_TILES) ) );
    list_vpos.push_back( Gtk::Menu_Helpers::MenuElem("선형", 
		sigc::bind( sigc::mem_fun(*this, &PrefWnd::OnMenuInterp), Gdk::INTERP_BILINEAR) ) );
    list_vpos.push_back( Gtk::Menu_Helpers::MenuElem("쌍곡선", 
		sigc::bind( sigc::mem_fun(*this, &PrefWnd::OnMenuInterp), Gdk::INTERP_HYPER) ) );

    interpType_ = preference_.InterpType();
    interpMenu_.set_active(interpType_);

    interpOptionMenu_.set_menu(interpMenu_);
    mainTable_.attach(interpOptionMenu_, 2, 3, 3, 4);

    // jpeg 압축률 슬라이더	
    jpegcompScale_.set_digits(0);
    jpegcompScale_.set_value_pos(Gtk::POS_TOP);
    jpegcompScale_.set_draw_value();
    jpegcompScale_.set_value(static_cast<double>(preference_.Jpegcomp()));	
    mainTable_.attach(baseJpegcompLabel_, 1, 2, 4, 5, Gtk::FILL);
    mainTable_.attach(jpegcompScale_, 2, 3, 4, 5);

    mainTable_.attach(dummyLabel1_, 0, 1, 1, 5, Gtk::SHRINK);
    mainTable_.attach(dummyLabel2_, 0, 3, 5, 6, Gtk::SHRINK);

    batchHeadLabel_.set_use_markup(true);
    mainTable_.attach(batchHeadLabel_, 0, 3, 6, 7, Gtk::FILL);

    // 에러 시 정지, 재귀처리 체크 버튼
    stoperrorBtn_.set_active(preference_.StopError());
    mainTable_.attach(stoperrorBtn_, 1, 3, 7, 8, Gtk::FILL);
    recursiveBtn_.set_active(preference_.Recursive());
    mainTable_.attach(recursiveBtn_, 1, 3, 8, 9, Gtk::FILL);

    // 화일 이름에 관한 컨트롤들
    mainTable_.attach(batchFileaddLabel_, 1, 2, 9, 10, Gtk::FILL);

    // 화일 이름 처리 방식에 관한 라디오 버튼
    Gtk::RadioButton::Group group = prefixBtn_.get_group();
    suffixBtn_.set_group(group);
    overwriteBtn_.set_group(group);
    prefixBtn_.signal_clicked().connect( 
	    sigc::bind( sigc::mem_fun(*this, &PrefWnd::OnBtnFileadd), FILEADD_PREFIX) );
    suffixBtn_.signal_clicked().connect( 
	    sigc::bind( sigc::mem_fun(*this, &PrefWnd::OnBtnFileadd), FILEADD_SUFFIX) );
    overwriteBtn_.signal_clicked().connect( 
	    sigc::bind( sigc::mem_fun(*this, &PrefWnd::OnBtnFileadd), FILEADD_OVERWRITE) );

    fileaddType_ = preference_.FileaddType();
    switch(fileaddType_)
    {
	case FILEADD_PREFIX:
	    prefixBtn_.set_active();
	    break;

	case FILEADD_SUFFIX:
	    suffixBtn_.set_active();
	    break;

	case FILEADD_OVERWRITE:
	    overwriteBtn_.set_active();
	    break;

	default:
	    prefixBtn_.set_active();			
    }
    fileaddRadioBox_.pack_start(prefixBtn_);
    fileaddRadioBox_.pack_start(suffixBtn_);
    fileaddRadioBox_.pack_start(overwriteBtn_);
    mainTable_.attach(fileaddRadioBox_, 2, 3, 9, 10);

    // 화일 이름 처리에 관한 입력 엔트리
    fileaddEntry_.set_text(preference_.FileaddString());		
    mainTable_.attach(fileaddEntry_, 2, 3, 10, 11);

    mainTable_.attach(dummyLabel3_, 0, 1, 7, 11, Gtk::SHRINK);
    mainTable_.attach(dummyLabel4_, 0, 3, 11, 12, Gtk::SHRINK);

    mainTable_.attach(btnBoxSeparator_, 0, 3, 12, 13, Gtk::FILL);

    previewTooltips_.set_tip(previewBtn_, "변환되는 이미지를 보여줍니다.\n"\
				"개별과 일괄 작업에 모두 적용됩니다.");
    extfilterTooltips_.set_tip(extFilterBtn_, "화일을 읽어서 이미지 화일을 "\
				"인식하지 않고, 확장자로 미리 구분합니다.\n\n" \
				"사용하지 않으면 작업이 느려집니다.");
    interpTooltips_.set_tip(interpOptionMenu_, "쌍곡선으로 갈수록 품질은 좋아지지만"\
				" 속도가 느려집니다.");
    jpegcompTooltips_.set_tip(jpegcompScale_, "jpeg 로 저장할 때 품질을"\
				" 설정합니다.\n\n높을 수록 품질은 좋지만"\
				" 화일 크기가 커집니다.");
    errstopTooltips_.set_tip(stoperrorBtn_, "일괄처리 작업 중 문제가 생기면"\
				" 멈춥니다.");
    recursiveTooltips_.set_tip(recursiveBtn_, "디렉토리를 읽을 때, 하위"\
				" 디렉토리까지 재귀적으로 읽습니다.\n\n디렉토리가"\
				" 복잡하다면 읽어오는데 시간이 많이 걸립니다.");
    prefixTooltips_.set_tip(prefixBtn_, "화일명 앞에 지정한 문장을 붙입니다.");
    suffixTooltips_.set_tip(suffixBtn_, "화일명 뒤에 지정한 문장을 붙입니다.");
    overwriteTooltips_.set_tip(overwriteBtn_, "화일명을 바꾸지 않고 덮어씁니다.\n\n"\
				"묻지 않고 덮어쓰게 되므로 주의하십시오.");

    saveBtn_.signal_clicked().connect(
	    sigc::mem_fun(*this, &PrefWnd::OnBtnSave) );
    mainBtnBox_.pack_start(saveBtn_);
    
    cancelBtn_.signal_clicked().connect(
	    sigc::mem_fun(*this, &PrefWnd::OnBtnCancel) );
    mainBtnBox_.pack_start(cancelBtn_);
    
    applyBtn_.signal_clicked().connect(
	    sigc::mem_fun(*this, &PrefWnd::OnBtnApply) );
    mainBtnBox_.pack_start(applyBtn_);		
    
    mainTable_.attach(mainBtnBox_, 0, 3, 13, 14, Gtk::FILL);

    show_all_children();
}

PrefWnd::~PrefWnd()
{
}

void PrefWnd::OnBtnCancel()
{
    hide();
}

void PrefWnd::OnBtnSave()
{
    string configfilePath = Glib::get_home_dir();
    configfilePath += "/.air";
    ofstream configfile(configfilePath.c_str());

    if(configfile) {
	configfile << *this;
    } else {
	Gtk::MessageDialog errDlg(*this, "설정화일을 저장할 수 없습니다.", false, Gtk::MESSAGE_ERROR);
	errDlg.run();
    }	
}

void PrefWnd::OnBtnApply()
{
    preference_ << *this;

    hide();
}

void PrefWnd::OnMenuInterp(Gdk::InterpType i)
{
    interpType_ = i;
}

void PrefWnd::OnBtnFileadd(FileAddType p)
{
    fileaddType_ = p;
    
    if(fileaddType_ == FILEADD_OVERWRITE) 
	fileaddEntry_.set_sensitive(false);
    else 
	fileaddEntry_.set_sensitive(true);
}

// operator<<(Preference&, const PrefWnd&) 는 양쪽 class에 friend 함수
void operator<<(Preference& preference, const PrefWnd& prefWnd)
{
    preference.preview_ = prefWnd.previewBtn_.get_active();
    preference.stopError_ = prefWnd.stoperrorBtn_.get_active();
    preference.extFilter_ = prefWnd.extFilterBtn_.get_active();
    preference.recursive_ = prefWnd.recursiveBtn_.get_active();
    preference.interpType_ = prefWnd.interpType_;
    preference.jpegcomp_ = static_cast<int>(prefWnd.jpegcompScale_.get_value());
    preference.fileaddType_ = prefWnd.fileaddType_;
    preference.fileaddString_ = prefWnd.fileaddEntry_.get_text();

}

// operator<<(ostream&, const PrefWnd&) 는 PrefWnd 에 friend 함수
void operator<<(ostream& os, const PrefWnd& prefWnd)
{
    os << "Preview = " << prefWnd.previewBtn_.get_active() << endl;
    os << "StopError = " << prefWnd.stoperrorBtn_.get_active() << endl;
    os << "ExtFilter = " << prefWnd.extFilterBtn_.get_active() << endl;
    os << "Recursive = " << prefWnd.recursiveBtn_.get_active() << endl;
    os << "InterpType = " << prefWnd.interpType_ << endl;
    os << "JpegComp = " << static_cast<int>(prefWnd.jpegcompScale_.get_value()) << endl;		
    os << "FileAddType = " << prefWnd.fileaddType_ << endl;
    os << "FileAddString = " << prefWnd.fileaddEntry_.get_text() << endl;

}
