// air is little image thumbnail program.
// Copyright (C) 2004 HG, Rhee <ez8tank@krpost.net>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
//
// All comments are korean, UTF-8 encoding.
//

#include <sstream>

#include "../include/singlescalewin.h"

using std::vector;
using std::string;
using std::stringstream;

SingleScaleWnd::SingleScaleWnd(const Preference& preference, 
	const string& filename, Glib::RefPtr<Gdk::Pixbuf> origimg)
: mainTable_(3, 16),
    typeRadioBox_(false, 5),
    entryBox_(false, 5),
    mainBtnBox_(Gtk::BUTTONBOX_END, 12),
    infoHeadLabel_("<span weight=\"bold\">정보</span>", 0.0, 0.5),
    infoFilenameLabel1_("화일 이름 : ", 0.0, 0.5),
    infoSizeLabel1_("이미지 사이즈 : ", 0.0, 0.5),
    setHeadLabel_("<span weight=\"bold\">설정</span>", 0.0, 0.5),
    setWidthLabel_("새로운 가로 : ", 0.0, 0.5),
    setHeightLabel_("새로운 세로 : ", 0.0, 0.5),
    setWidthRatioLabel_("가로 비율 : ", 0.0, 0.5),
    setHeightRatioLabel_("세로 비율 : ", 0.0, 0.5),
    setTypeLabel_("저장할 형식 : ", 0.0, 0.5),
    setFilenameLabel_("화일명 : ", 0.0, 0.5),
    sameScaleBtn_("가로 세로 같은 비율"),
    browseBtn_("찾기"),
    cancelBtn_(Gtk::Stock::CANCEL),
    okBtn_(Gtk::Stock::OK),
    widthAdj_(1.0, 1.0, 10000.0, 1.0, 5.0, 0.0),
    heightAdj_(1.0, 1.0, 10000.0, 1.0, 5.0, 0.0),
    widthRatioAdj_(1.0, 0.0, 100.0, 0.01, 5.0, 0.0),
    heightRatioAdj_(1.0, 0.0, 100.0, 0.01, 5.0, 0.0),
    preference_(preference),
    origImage_(origimg)
{
    set_title("설정 창");
    set_border_width(12);
    set_resizable(false);
    set_modal(true);

    origFile_.reserve(3);
    SplitPath(filename, origFile_);

    mainTable_.set_row_spacings(6);
    mainTable_.set_col_spacing(0, 12);
    mainTable_.set_col_spacing(1, 12);
    add(mainTable_);

    // /------------------------\
    // | label                  |
    // | | PreviewArea          |
    // | | label  | label       |
    // | | label  | label       |
    // |------------------------|
    // | label                  |
    // | | chk btn              |
    // | | label  | spin btn    |
    // | | label  | spin btn    |
    // | | label  | spin btn    |
    // | | label  | spin btn    |
    // | | label  | radio btn   |
    // | | label  | entry | btn |
    // |------------------------|
    // | hSeparator             |
    // | hBtnBox                |
    // \------------------------/

    infoHeadLabel_.set_use_markup(true);
    mainTable_.attach(infoHeadLabel_, 0, 3, 0, 1, Gtk::FILL);

    if(preference_.Preview()) {
	previewArea_ = manage(new PreviewArea);
	previewArea_->set_size_request(100, 100);
	mainTable_.attach(*previewArea_, 0, 3, 1, 2, Gtk::FILL);
    } else {
	mainTable_.attach(*manage(new Gtk::Label), 0, 3, 1, 2, Gtk::FILL);
    }

    mainTable_.attach(infoFilenameLabel1_, 1, 2, 2, 3, Gtk::FILL);

    Glib::ustring filenameForLabel;
    try {
	filenameForLabel = Glib::locale_to_utf8(origFile_[1]);
    } catch(Glib::ConvertError& err) {
	filenameForLabel = origFile_[1];
    }

    if(origFile_.size() == 3) 
	filenameForLabel += origFile_[2];
	
    infoFilenameLabel2_.set_text(filenameForLabel);	
    infoFilenameLabel2_.set_alignment(0.0, 0.5);
    mainTable_.attach(infoFilenameLabel2_, 2, 3, 2, 3, Gtk::FILL);

    mainTable_.attach(infoSizeLabel1_, 1, 2, 3, 4, Gtk::FILL);

    stringstream buf;
    buf << origImage_->get_width() << " x " << origImage_->get_height();		
    infoSizeLabel2_.set_text(buf.str());
    infoSizeLabel2_.set_alignment(0.0, 0.5);
    mainTable_.attach(infoSizeLabel2_, 2, 3, 3, 4, Gtk::FILL);

    mainTable_.attach(dummyLabel1_, 0, 1, 1, 4, Gtk::SHRINK);
    mainTable_.attach(dummyLabel2_, 0, 3, 4, 5, Gtk::SHRINK);	

    setHeadLabel_.set_use_markup(true);
    mainTable_.attach(setHeadLabel_, 0, 3, 5, 6, Gtk::FILL);

    sameScaleBtn_.set_active();
    sameScaleBtn_.signal_clicked().connect(
	    sigc::mem_fun(*this, &SingleScaleWnd::OnCheckBtnSamescale) );
    mainTable_.attach(sameScaleBtn_, 1, 3, 6, 7, Gtk::FILL);

    newWidthSpin_.set_adjustment(widthAdj_);
    newWidthSpin_.set_value(origImage_->get_width());
    newWidthSpin_.signal_value_changed().connect(
	    sigc::mem_fun(*this, &SingleScaleWnd::OnSpinBtnWidth) );
    mainTable_.attach(setWidthLabel_, 1, 2, 7, 8, Gtk::FILL);
    mainTable_.attach(newWidthSpin_, 2, 3, 7, 8, Gtk::FILL);

    newHeightSpin_.set_adjustment(heightAdj_);
    newHeightSpin_.set_value(origImage_->get_height());
    newHeightSpin_.signal_value_changed().connect(
	    sigc::mem_fun(*this, &SingleScaleWnd::OnSpinBtnHeight) );
    mainTable_.attach(setHeightLabel_, 1, 2, 8, 9, Gtk::FILL);
    mainTable_.attach(newHeightSpin_, 2, 3, 8, 9, Gtk::FILL);

    xScaleSpin_.set_adjustment(widthRatioAdj_);
    xScaleSpin_.set_digits(2);
    xScaleSpin_.signal_value_changed().connect(
	    sigc::mem_fun(*this, &SingleScaleWnd::OnSpinBtnX) );
    mainTable_.attach(setWidthRatioLabel_, 1, 2, 9, 10, Gtk::FILL);
    mainTable_.attach(xScaleSpin_, 2, 3, 9, 10, Gtk::FILL);

    yScaleSpin_.set_adjustment(heightRatioAdj_);
    yScaleSpin_.set_digits(2);
    yScaleSpin_.signal_value_changed().connect(
	    sigc::mem_fun(*this, &SingleScaleWnd::OnSpinBtnY) );
    mainTable_.attach(setHeightRatioLabel_, 1, 2, 10, 11, Gtk::FILL);
    mainTable_.attach(yScaleSpin_, 2, 3, 10, 11, Gtk::FILL);

    mainTable_.attach(setTypeLabel_, 1, 2, 11, 12, Gtk::FILL);
    Gtk::RadioButton::Group group;
    for(vector<Gdk::PixbufFormat>::const_iterator i = preference_.WritableFormats().begin(); 
	    i != preference_.WritableFormats().end(); ++i) {
	if(ext_.empty()) ext_ = i->get_name();
	Gtk::RadioButton* rb = manage(new Gtk::RadioButton(group, i->get_name()));
	rb->signal_clicked().connect( sigc::bind(
		    sigc::mem_fun(*this, &SingleScaleWnd::OnRadioBtnExt), i->get_name() ) );
	typeRadioBox_.pack_start(*rb);
    }
    mainTable_.attach(typeRadioBox_, 2, 3, 11, 12, Gtk::FILL);

    mainTable_.attach(setFilenameLabel_, 1, 2, 12, 13, Gtk::FILL);

    Glib::ustring filenameForEntry;
    try {
	filenameForEntry = Glib::locale_to_utf8(preference_.ConvertFilename(origFile_[1]));
	filenameForEntry += ".";
	filenameForEntry += ext_;
    } catch(Glib::ConvertError& err) {
	filenameForEntry = preference_.ConvertFilename(origFile_[1]);
	filenameForEntry += "."; 
	filenameForEntry += ext_;
    }
    filenameEntry_.set_text(filenameForEntry);

    entryBox_.pack_start(filenameEntry_, Gtk::PACK_SHRINK);

    browseBtn_.signal_clicked().connect( 
	    sigc::mem_fun(*this, &SingleScaleWnd::OnBtnBrowse) );
    entryBox_.pack_start(browseBtn_, Gtk::PACK_SHRINK);	
    mainTable_.attach(entryBox_, 2, 3, 12, 13, Gtk::FILL);

    mainTable_.attach(dummyLabel3_, 0, 1, 6, 13, Gtk::SHRINK);
    mainTable_.attach(dummyLabel4_, 0, 3, 13, 14, Gtk::SHRINK);

    mainTable_.attach(btnBoxSeparator_, 0, 3, 14, 15, Gtk::FILL);

    cancelBtn_.signal_clicked().connect(
	    sigc::mem_fun(*this, &SingleScaleWnd::OnBtnCancel) );
    mainBtnBox_.pack_start(cancelBtn_);
    okBtn_.signal_clicked().connect(
	    sigc::mem_fun(*this, &SingleScaleWnd::OnBtnOk) );
    mainBtnBox_.pack_start(okBtn_);
    
    mainTable_.attach(mainBtnBox_, 0, 3, 15, 16, Gtk::FILL);

    show_all_children();

    if(preference_.Preview()) {
	previewArea_->SetImg(origImage_);
	previewArea_->Draw();
    }
}

SingleScaleWnd::~SingleScaleWnd()
{
}

void SingleScaleWnd::OnSpinBtnWidth()
{	
    xScaleSpin_.set_value(newWidthSpin_.get_value() / origImage_->get_width());
}

void SingleScaleWnd::OnSpinBtnHeight()
{
    yScaleSpin_.set_value(newHeightSpin_.get_value() / origImage_->get_height());
}

void SingleScaleWnd::OnSpinBtnX()
{
    if(sameScaleBtn_.get_active()) {
	newHeightSpin_.set_value(origImage_->get_height() * xScaleSpin_.get_value());
    }
    newWidthSpin_.set_value(origImage_->get_width() * xScaleSpin_.get_value());
}

void SingleScaleWnd::OnSpinBtnY()
{
    if(sameScaleBtn_.get_active()) {
	newWidthSpin_.set_value(origImage_->get_width() * yScaleSpin_.get_value());
    }
    newHeightSpin_.set_value(origImage_->get_height() * yScaleSpin_.get_value());
}

void SingleScaleWnd::OnCheckBtnSamescale()
{
    if(sameScaleBtn_.get_active())
	yScaleSpin_.set_value(xScaleSpin_.get_value());		
}

void SingleScaleWnd::OnRadioBtnExt(const string& p)
{
    ext_ = p;

    Glib::ustring buf = StripExt(filenameEntry_.get_text());
    buf += ".";

    if(ext_ == "jpeg") 
	buf += "jpg";
    else 
	buf += ext_;

    filenameEntry_.set_text(buf);	

}

void SingleScaleWnd::OnBtnBrowse()
{
    Gtk::FileSelection fileDlg("저장할 화일");
    fileDlg.set_transient_for(*this);
    fileDlg.set_filename(origFile_[0] + filenameEntry_.get_text());

    int result = fileDlg.run();
    fileDlg.hide();

    switch(result) {
	case(Gtk::RESPONSE_OK):
	    {
		string tempFile;
		try {
		    tempFile = Glib::locale_to_utf8(fileDlg.get_filename());
		} catch(Glib::ConvertError& err) {
		    tempFile = fileDlg.get_filename();
		}
		if(IsDirectory(tempFile) != 1) {
		    vector<string>().swap(origFile_);
		    SplitPath(tempFile, origFile_);
		    filenameEntry_.set_text(origFile_[1] + origFile_[2]);
		}
		break;	
	    }
	case(Gtk::RESPONSE_CANCEL):
	    {
		break;
	    }	
    }	
}

void SingleScaleWnd::OnBtnCancel()
{
    hide();
}

void SingleScaleWnd::OnBtnOk()
{
    string filename = origFile_[0];
    try {
	filename += Glib::locale_from_utf8(filenameEntry_.get_text());
    } catch(Glib::ConvertError& err) {
	filename += filenameEntry_.get_text();
    }
    if(Glib::file_test(filename, Glib::FILE_TEST_EXISTS)) {
	Gtk::MessageDialog errDlg(*this, "화일이 이미 존재합니다.\n 덮어 쓰시겠습니까?", false, Gtk::MESSAGE_QUESTION,
		Gtk::BUTTONS_OK_CANCEL );
	int rtn = errDlg.run();
	switch(rtn)
	{
	    case(Gtk::RESPONSE_OK):
		{
		    SaveFile();
		    hide();
		    break;
		}
	    case(Gtk::RESPONSE_CANCEL):
		{
		    break;
		}
	}		
    } else {
	SaveFile();
	hide();
    }

}

void SingleScaleWnd::SaveFile()
{
    Glib::RefPtr<Gdk::Pixbuf> dest = 
	origImage_->scale_simple(newWidthSpin_.get_value_as_int(),
		newHeightSpin_.get_value_as_int(),
		preference_.InterpType());

    string filename = origFile_[0];
    try {
	filename += Glib::locale_from_utf8(filenameEntry_.get_text());
    } catch(Glib::ConvertError& err) {
	filename += filenameEntry_.get_text();
    }

    try {
	if(ext_ == "jpeg") {
	    dest->save(filename, "jpeg",
		    preference_.JpegOptionKeys(), preference_.JpegOptionValues());
	}
	else dest->save(filename, ext_);

    } catch(Gdk::PixbufError& err) {
	Gtk::MessageDialog errDlg(*this, err.what(), false, Gtk::MESSAGE_ERROR);
	errDlg.run();	
    } catch(Glib::FileError& err) {
	Gtk::MessageDialog errDlg(*this, err.what(), false, Gtk::MESSAGE_ERROR);
	errDlg.run();
    }

}
